@@ -292,8 +292,11 @@ def kill_atx_agent(device):
 
 
 def random_sleep(inf=0.5, sup=3.0, modulable=True, logging=True):
+    MIN_INF = 0.3
     multiplier = float(args.speed_multiplier)
     delay = uniform(inf, sup) / (multiplier if modulable else 1.0)
+    if delay < MIN_INF:
+        delay = MIN_INF
     if logging:
         logger.debug(f"{str(delay)[0:4]}s sleep")
     sleep(delay)
@@ -357,6 +360,24 @@ def stop_bot(device, sessions, session_state, screen_record, was_sleeping=False)
     sys.exit(0)
 
 
+def can_repeat(current_session, max_sessions):
+    if max_sessions != -1:
+        logger.info(
+            f"You completed {current_session} session(s). {max_sessions-current_session} session(s) left.",
+            extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
+        )
+        if current_session >= max_sessions:
+            logger.info(
+                "You reached the total-sessions limit! Finish.",
+                extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
+            )
+            return False
+        else:
+            return True
+    else:
+        return True
+
+
 def get_value(count, name, default):
     def print_error():
         logger.error(
